# Automatically generated. All your changes will be lost on recompilation!

"""
Optical solvers.

This package combines all optical solvers and helper functions that can
create solver classes without a need to explicitly import the proper binary
module.

Example:

    >>> import optical
    >>> optical.BesselCyl('mysolver')
    <optical.modal.BesselCyl at 0x42ac2b8>

Solver classes
--------------

.. autosummary::
   :toctree: optical
   :template: solver.rst

   effective.EffectiveIndex2D
   effective.EffectiveFrequencyCyl
   modal.Fourier2D
   modal.Fourier3D
   modal.BesselCyl
"""


def EffectiveIndex2D(name=''):
    """
    Create EffectiveIndex2D solver.

    This function automatically loads ``effective`` submodule and creates
    :class:`~effective.EffectiveIndex2D` class.

    Args:
        name (str): Solver name.
    """
    import optical.effective
    return optical.effective.EffectiveIndex2D(name)


def EffectiveFrequencyCyl(name=''):
    """
    Create EffectiveFrequencyCyl solver.

    This function automatically loads ``effective`` submodule and creates
    :class:`~effective.EffectiveFrequencyCyl` class.

    Args:
        name (str): Solver name.
    """
    import optical.effective
    return optical.effective.EffectiveFrequencyCyl(name)


def Fourier2D(name=''):
    """
    Create Fourier2D solver.

    This function automatically loads ``modal`` submodule and creates
    :class:`~modal.Fourier2D` class.

    Args:
        name (str): Solver name.
    """
    import optical.modal
    return optical.modal.Fourier2D(name)


def Fourier3D(name=''):
    """
    Create Fourier3D solver.

    This function automatically loads ``modal`` submodule and creates
    :class:`~modal.Fourier3D` class.

    Args:
        name (str): Solver name.
    """
    import optical.modal
    return optical.modal.Fourier3D(name)


def BesselCyl(name=''):
    """
    Create BesselCyl solver.

    This function automatically loads ``modal`` submodule and creates
    :class:`~modal.BesselCyl` class.

    Args:
        name (str): Solver name.
    """
    import optical.modal
    return optical.modal.BesselCyl(name)
